package org.jeecg.modules.ctop.controller;

import cn.com.ctop.common.utils.Check;
import cn.com.ctop.common.utils.HttpUtils;
import cn.com.ctop.common.utils.PropertiesUtils;
import cn.com.ctop.kuaishou.modules.material.entity.KuaishouAccessToken;
import cn.com.ctop.kuaishou.modules.material.entity.KuaishouResultToken;
import cn.com.ctop.toutiao.common.BytedanceInterfaceConstant;
import com.alibaba.fastjson.JSONObject;
import constant.KuaishouInterfaceConstant;
import org.apache.commons.lang.StringUtils;
import org.jeecg.modules.ctop.entity.ByteDanceAdvertiser;
import org.jeecg.modules.ctop.entity.CTopOauthToken;
import org.jeecg.modules.ctop.entity.UserAllocation;
import org.jeecg.modules.ctop.mapper.CTopOauthTokenMapper;
import org.jeecg.modules.ctop.mapper.UserAllocationMapper;
import org.jeecg.modules.ctop.service.IBindAccountAuthService;
import org.jeecg.modules.ctop.service.IByteDanceAdvertiserDataService;
import org.jeecg.modules.kuaishou.service.IKuaishouInterfaceService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

@Controller
public class CallbackController {
    private static final Logger logger = LoggerFactory.getLogger(CallbackController.class);

    @Autowired
    private IKuaishouInterfaceService kuaishouInterfaceService;

    @Autowired
    private CTopOauthTokenMapper cTopOauthTokenMapper;
    @Autowired
    private IBindAccountAuthService bindAccountAuthService;

    @Autowired
    private UserAllocationMapper allocationMapper;


    @GetMapping("/kuaishou")
    @ResponseBody
    public String kuaishou(HttpServletRequest request,
                           HttpServletResponse response,
                           @RequestParam("auth_code") String authCode,
                           @RequestParam("state") String state) {
        try {
            JSONObject json = JSONObject.parseObject(state);
            if (Check.isNull(json)) {
                throw new Exception("回调信息为空");
            }
            if (StringUtils.isNotBlank(authCode)) {
                KuaishouResultToken accessToken = kuaishouInterfaceService.getAccessToken(authCode);
                if (accessToken.getCode() != 0) {
                    throw new Exception("快手回调token返回错误");
                }
                KuaishouAccessToken token = accessToken.getData();
                if (token != null) {
                    Long accountId = token.getAccountId();
                    String advertiser_id = json.getString("advertiserId");
                    String distributionName = json.getString("distributionName");//指定分配人姓名
                    String advertiserName = json.getString("advertiser_name"); // 广告主名称
                    String mediaId = json.getString("mediaId"); // 媒体类型： 1 头条 2 快手
                    String realName = json.getString("realName");
                    String departmentId = json.getString("departmentId"); //部门id
                    String userId = json.getString("userId"); //创建人
                    UserAllocation userAllocation = new UserAllocation();
                    userAllocation.setDistributionName(distributionName);
                    userAllocation.setUserId(userId);
                    userAllocation.setAdvertiserId(advertiser_id);
                    userAllocation.setAuthName(json.getString("authName"));
                    userAllocation.setAdvertiserName(advertiserName);
                    userAllocation.setMediaId(mediaId);
                    userAllocation.setDepartmentId(departmentId);
                    userAllocation.setOperationName(realName);
                    userAllocation.setAccountId(accountId);
                    userAllocation.setCreateTime(new Date());
                    userAllocation.setUpdateTime(new Date());
                    allocationMapper.deleteById(userAllocation.getId());
                    Map<String, Object> deleteUserMap = new HashMap<>();
                    deleteUserMap.put("advertiser_id", advertiser_id);
                    deleteUserMap.put("account_id", accountId);
                    allocationMapper.deleteByMap(deleteUserMap);
                    int i = allocationMapper.insert(userAllocation);
                    if (i > 0) {
                        logger.info("同步分配用户表完成,task_id:{},accountId:{}", state, accountId);
                    }
                    CTopOauthToken topOauthToken = new CTopOauthToken();
                    Map<String, Object> deleteMap = new HashMap<>();
                    deleteMap.put("advertiser_id", advertiser_id);
                    deleteMap.put("account_id", accountId);
                    cTopOauthTokenMapper.deleteByMap(deleteMap); //删除 广告主id下的相关授权信息
                    topOauthToken.setAccountId(accountId);
                    topOauthToken.setMediaId("2");
                    topOauthToken.setAdvertiserId(advertiser_id);
                    topOauthToken.setAccessToken(token.getAccessToken());
                    long accessTokenExpireIn = token.getAccessTokenExpiresIn() * 1000L;
                    long now = System.currentTimeMillis();
                    Date accessTokenExpireInDate = new Date(now + accessTokenExpireIn);
                    topOauthToken.setAccessTokenExpiresIn(accessTokenExpireInDate);
                    topOauthToken.setRefreshToken(token.getRefreshToken());
                    long refreshTokenExpireIn = token.getRefreshTokenExpiresIn() * 1000L;
                    Date refreshTokenExpireInDate = new Date(now + refreshTokenExpireIn);
                    topOauthToken.setRefreshTokenExpiresIn(refreshTokenExpireInDate);
                    topOauthToken.setUpdateTime(new Date());
                    cTopOauthTokenMapper.insert(topOauthToken);
                    bindAccountAuthService.addBindAccount(accountId, KuaishouInterfaceConstant.LOGIN_TYPE_KUAISHOU, advertiser_id); //账号绑定
                }
                return "auth_success";
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return "auth_fail";
    }


    @RequestMapping("/bytedance")
    @ResponseBody
    public String bytedance(HttpServletRequest request,
                            HttpServletResponse response,
                            @RequestParam("auth_code") String authCode,
                            @RequestParam("state") String state) {

        if (StringUtils.isNotBlank(authCode)) {
            Map<String, Object> map = this.getByteDanceAccessToken(authCode, state);
            Integer code = (Integer) map.get("code");
            if (code == 0) {
                return "auth_success";
            }
        }
        return "auth_fail";
    }

    @Autowired
    private IByteDanceAdvertiserDataService advertiserDataService;

    public Map<String, Object> getByteDanceAccessToken(String authCode, String state) {
        Map<String, Object> param = new HashMap<String, Object>();
        param.put("app_id", PropertiesUtils.getValue("bytedance_config", "bytedance_appid"));
        param.put("secret", PropertiesUtils.getValue("bytedance_config", "bytedance_secret"));
        param.put("grant_type", "auth_code");
        param.put("auth_code", authCode);
        logger.info("state:{}", state);
        Map<String, Object> returnMap = new HashMap<>();
        try {
            JSONObject json = JSONObject.parseObject(state);
            if (Check.isNull(json)) {
                throw new Exception("回调信息为空");
            }
            String result = HttpUtils.httpPostRequest(PropertiesUtils.getValue("bytedance_config", "bytedance_api_url") + BytedanceInterfaceConstant.AUTH_TOKEN, param, new HashMap<>());
            JSONObject resultObject = JSONObject.parseObject(result);
            Integer code = resultObject.getInteger("code");
            String message = resultObject.getString("message");
            if (null == code || code != 0) {
                logger.info("获取token失败,state:{},message:{}", state, message);
                throw new Exception("token返回码异常");
            }
            String advertiser_id = json.getString("advertiserId");//广告主id
            String mediaId = json.getString("mediaId"); // 媒体类型： 1 头条 2 快手
            String realName = json.getString("realName");
            String departmentId = json.getString("departmentId"); //部门id
            String distributionName = json.getString("distributionName");//指定分配人姓名
            String advertiserName = json.getString("advertiser_name"); // 广告主名称
            String userId = json.getString("userId"); //创建人

            JSONObject data = resultObject.getJSONObject("data");
            CTopOauthToken token = new CTopOauthToken(advertiser_id, data);
            Map<String, Object> deleteMap = new HashMap<>();
            deleteMap.put("advertiser_id", advertiser_id);
            deleteMap.put("account_id", token.getAccountId());
            cTopOauthTokenMapper.deleteByMap(deleteMap); //删除 广告主id下的相关授权信息
            cTopOauthTokenMapper.insert(token);
            //获取广告主信息
            Map<String, Object> advertiserDataMap = advertiserDataService.getAdvertiserInfo(token.getAccountId() + "");
            Boolean getSuccess = (Boolean) advertiserDataMap.get("success");
            if (null == getSuccess || !getSuccess) {
                logger.info("获取广告主信息失败,accountId:{},message:{}", token.getAccountId(), (String) advertiserDataMap.get("message"));
                throw new Exception("获取广告主信息失败");
            }
            ByteDanceAdvertiser advertiserData = (ByteDanceAdvertiser) advertiserDataMap.get("data");
            UserAllocation userAllocation = new UserAllocation();
            userAllocation.setDistributionName(distributionName);
            userAllocation.setUserId(userId);
            userAllocation.setDepartmentId(departmentId);
            userAllocation.setOperationName(realName);
            userAllocation.setAccountId(token.getAccountId());
            userAllocation.setAdvertiserName(advertiserName);
            userAllocation.setMediaId(mediaId);
            userAllocation.setAuthName(advertiserData.getName());
            userAllocation.setAdvertiserId(advertiser_id);
            userAllocation.setCreateTime(new Date());
            userAllocation.setUpdateTime(new Date());
            allocationMapper.deleteById(userAllocation.getId());
            Map<String, Object> deleteUserMap = new HashMap<>();
            deleteUserMap.put("advertiser_id", advertiser_id);
            deleteUserMap.put("account_id", token.getAccountId());
            allocationMapper.deleteByMap(deleteUserMap);
            int i = allocationMapper.insert(userAllocation);
            if (i > 0) {
                logger.info("同步分配用户表完成,task_id:{},accountId:{}", state, token.getAccountId());

            }
            bindAccountAuthService.addBindAccount(token.getAccountId(), KuaishouInterfaceConstant.LOGIN_TYPE_BYTEDANCE, advertiser_id); //账号绑定
            returnMap.put("code", 0);
            returnMap.put("desc", "success");
        } catch (Exception e) {
            e.printStackTrace();
            returnMap.put("code", -1);
            returnMap.put("desc", "回调token异常");

        }
        return returnMap;
    }

}
